import os
import random

import pygame
pygame.init()

import settings
import helpers
from Room import Room
from Jungleguy import Jungleguy
from ClimbingPlant import ClimbingPlant
from Platform import Platform
from Turbine import Turbine
from Firefly import Firefly
from Lamp import Lamp
from Cloud import Cloud
from RootArea import RootArea
from Stats import Stats
from SingleStat import SingleStat
from Menu import Menu


class JungleReconstruction( object ):
	"""Entry point of the game.

	"""
	def __init__( self ):
		"""

		"""
		self.window = pygame.display.set_mode( settings.WINDOW_DIMENSIONS )
		self.clock = pygame.time.Clock()

		self.bg_img = pygame.image.load( os.path.join( os.getcwd(), "gfx", "night.png" ) )
		self.bg_rect = self.bg_img.get_rect()


	def prepare_objects( self ):

		self.jungleguy = Jungleguy( **settings.JUNGLEGUY )

		self.climbing_plant = ClimbingPlant( ( 320, 480 ) )

		self.turbine = Turbine( **settings.TURBINE_1 )
		self.turbine_2 = Turbine( **settings.TURBINE_2 )

		self.fireflies = pygame.sprite.Group()
		for x in range( settings.N_FIREFLIES ):
			self.fireflies.add(
				Firefly( **settings.FIREFLY )
			)


		self.lamp = Lamp( **settings.LAMP )

		self.light_img = helpers.load_image( "light.png" )
		self.light_img.set_alpha( 175 )	
		self.light_rect = self.light_img.get_rect()
		self.light_rect.center = ( 319, 240 )
		self.display_light = False

		self.clouds = pygame.sprite.Group()
		for x in range( settings.N_CLOUDS ):
			self.clouds.add(
				Cloud( **settings.CLOUD )
			)

		# Rootarea
		self.rootarea = RootArea( **settings.ROOTAREA )

		# Raindrops
		self.raindrops = pygame.sprite.Group()

		# Stats
		self.stats = Stats()

		# Timer
		self.timer_stat = SingleStat( **settings.SINGLE_STAT_TIMER )
		self.timer_counter = 0
		self.timer_counter_max = settings.FPS
		self.add_to_time = settings.FPS / float(1000)
		self.seconds_passed = 0


	def run_game( self ):
		"""

		"""	
		self.run_intro()
		self.run_main_menu()
		pygame.display.quit()


	def run_intro( self ):
		"""

		"""
		running = True

		intro1_img = pygame.image.load( os.path.join( os.getcwd(), "gfx", "story_1.png" ) )
		intro1_rect = self.bg_img.get_rect()

		intro2_img = pygame.image.load( os.path.join( os.getcwd(), "gfx", "story_2.png" ) )
		intro2_rect = self.bg_img.get_rect()

		timer1 = 0
		timer1_max = settings.FPS * 6

		helpers.load_music( "happy.ogg" )
		pygame.mixer.music.play()
		
		while running:
			eventlist = pygame.event.get()
			for event in eventlist:
				if event.type == pygame.KEYDOWN and event.key == pygame.K_RETURN:
					running = False

			self.window.blit( intro1_img, intro1_rect )
			
			pygame.display.update()
			self.clock.tick( settings.FPS )

			if timer1 >= timer1_max:
				running = False
			else:
				timer1 += 1


		timer1 = 0
		timer1_max = settings.FPS * 13			

		running = True

		track1 = helpers.load_music( "sad.ogg" )
		pygame.mixer.music.play()

		while running:
			eventlist = pygame.event.get()
			for event in eventlist:
				if event.type == pygame.KEYDOWN and event.key == pygame.K_RETURN:
					running = False

			self.window.blit( intro2_img, intro2_rect )	

			pygame.display.update()
			self.clock.tick( settings.FPS )

			if timer1 >= timer1_max:
				running = False
			else:
				timer1 += 1

		track1 = helpers.load_music( "track1.ogg" )
		pygame.mixer.music.set_volume( 0.5 )
		pygame.mixer.music.play( -1 )			


	def run_main_menu( self ):
		"""

		"""
		main_menu = Menu( settings.MAIN_MENU )		

		running = True
		
		while running:
			eventlist = pygame.event.get()
			for event in eventlist:
				if event.type == pygame.KEYDOWN and event.key == pygame.K_ESCAPE:
					running = False

			ret = main_menu.update( eventlist )
			if ret == "start":
				self.run()
				if self.success:
					self.run_game_over()
			elif ret == "end":
				running = False

			self.window.blit( self.bg_img, self.bg_rect )

			main_menu.draw( self.window )		

			pygame.display.update()
			self.clock.tick( settings.FPS )	


	def run( self ):
		"""Runs the game.

		"""

		self.prepare_objects()

		self.running = True
		self.success = False
		self.skip_updates = False
		
		while self.running:
			eventlist = pygame.event.get()
			for event in eventlist:
				if event.type == pygame.KEYDOWN and event.key == pygame.K_ESCAPE:
					if self.run_pause_menu():
						self.running = False
						self.skip_updates = True

			if self.skip_updates:
				continue

			self.jungleguy.update( eventlist )
			self.turbine.update( self.jungleguy, self.lamp )
			self.turbine_2.update( self.jungleguy, self.lamp )
			self.fireflies.update( self.jungleguy )			
			self.climbing_plant.update( self.lamp, self.rootarea, self.jungleguy )
			self.lamp.update()
			self.clouds.update( self.jungleguy, self.raindrops )
			self.raindrops.update()
			self.rootarea.update( self.raindrops )

			if self.check_create_new_firefly():
				self.create_new_firefly()

			if self.check_create_new_cloud():
				self.create_new_cloud()

			if self.check_plant_has_reached_lamp():
				self.running = False
				self.success = True

			self.stats.update(
				self.rootarea.water_in_soil,
				self.lamp.energy,
				self.jungleguy.fireflies,
				self.jungleguy
			)

			self.seconds_passed += self.add_to_time
			self.timer_stat.update( self.seconds_passed )
											
			self.window.blit( self.bg_img, self.bg_rect )
			self.window.blit( self.turbine.image, self.turbine.rect )
			self.window.blit( self.turbine_2.image, self.turbine_2.rect )
			self.clouds.draw( self.window )			
			self.fireflies.draw( self.window )
			if self.lamp.has_energy():
				self.window.blit( self.light_img, self.light_rect )	
			self.window.blit( self.lamp.image, self.lamp.rect )	
	
			self.window.blit( self.rootarea.image, self.rootarea.rect )			
			for ppart in self.climbing_plant.plant_parts:
				self.window.set_at( ppart, settings.GREEN )

			self.raindrops.draw( self.window )
				
			self.window.blit( self.jungleguy.image, self.jungleguy.rect )

			self.window.blit( self.stats.single_stat_water.image,
							  self.stats.single_stat_water.rect )
			self.window.blit( self.stats.single_stat_elec.image,
							  self.stats.single_stat_elec.rect )
			self.window.blit( self.stats.single_stat_fireflies.image,
							  self.stats.single_stat_fireflies.rect )
			self.window.blit( self.stats.single_stat_jump_energy.image,
							  self.stats.single_stat_jump_energy.rect )
			self.window.blit( self.stats.single_stat_straight_jump_energy.image,
							  self.stats.single_stat_straight_jump_energy.rect )
			self.window.blit( self.stats.single_stat_movement_energy.image,
							  self.stats.single_stat_movement_energy.rect )

			self.window.blit( self.timer_stat.image,
							  self.timer_stat.rect )							  

			
			pygame.display.update()
			self.clock.tick( settings.FPS )


	def run_pause_menu( self ):
		"""

		"""
		pause_menu = Menu( settings.PAUSE_MENU )

		running = True
		
		while running:
			eventlist = pygame.event.get()
			for event in eventlist:
				if event.type == pygame.KEYDOWN and event.key == pygame.K_ESCAPE:
					running = False

			ret = pause_menu.update( eventlist )
			if ret == "yes":
				running = False
				return True
			if ret == "no":
				running = False

			self.window.blit( self.bg_img, self.bg_rect )

			pause_menu.draw( self.window )		

			pygame.display.update()
			self.clock.tick( settings.FPS )	
			

	def run_game_over( self ):
		"""

		"""

		game_over_menu = Menu( settings.GAME_OVER_MENU )

		your_time = SingleStat( **settings.YOUR_TIME )
		your_time.update( self.seconds_passed )
		running = True
		
		while running:
			eventlist = pygame.event.get()
			for event in eventlist:
				if event.type == pygame.KEYDOWN and event.key == pygame.K_ESCAPE:
					running = False

			if game_over_menu.update( eventlist ) == "ok":
				running = False

			self.window.blit( self.bg_img, self.bg_rect )
			self.window.blit( your_time.image, your_time.rect )	

			game_over_menu.draw( self.window )		

			pygame.display.update()
			self.clock.tick( settings.FPS )					


	def check_create_new_firefly( self ):
		"""

		"""
		if len( self.fireflies ) < settings.N_FIREFLIES:
			return True
		else:
			return False


	def create_new_firefly( self ):
		"""

		"""
		self.fireflies.add(
			Firefly(
				**settings.FIREFLY )
			)


	def check_create_new_cloud( self ):
		"""

		"""
		if len( self.clouds ) < settings.N_CLOUDS:
			return True
		else:
			return False


	def create_new_cloud( self ):
		"""

		"""
		self.clouds.add(
			Cloud(
				**settings.CLOUD
			)
		)


	def check_plant_has_reached_lamp( self ):
		"""

		"""
		if len( self.climbing_plant.plant_parts ) > 0:
			top_left = self.climbing_plant.plant_parts[-1]
			rect = pygame.Rect( ( top_left[0] - 1, top_left[1] - 1, top_left[0], top_left[1] ) )
			if rect.colliderect( self.lamp.rect ):
				return True
		return False


	def game_over( self ):
		self.running = False
