import pygame
pygame.init()


WINDOW_WIDTH = 640
WINDOW_HEIGHT = 480
WINDOW_DIMENSIONS = ( WINDOW_WIDTH, WINDOW_HEIGHT )
WINDOW_FLAGS = ( pygame.FULLSCREEN, )
FPS = 30

BLUE = pygame.Color( 0, 100, 250, 0 )
WHITE = pygame.Color( 255, 255, 255, 0 )
GREEN = pygame.Color( 0, 255, 0, 0 )
DARK_GREEN = pygame.Color( 0, 180, 0, 0 )
GREY = pygame.Color( 120, 120, 120, 0 )
YELLOW = pygame.Color( 230, 230, 0, 0 )

TILE_SIZE = 20

GFX_DIR = "gfx"
SOUND_DIR = "sfx"
MUSIC_DIR = "music"
LEVEL_DIR = "levels"
FONT_DIR = "font"


TILE_DICT = {
	"#": "blank.png",
	"1": "grass_1.png",
	"2": "grass_2.png"
}

JUNGLEGUY = {
	"_images_right_dir": ( "jungleguy_right_1.png", "jungleguy_right_2.png" ),
	"_images_left_dir": ( "jungleguy_left_1.png", "jungleguy_left_2.png" ),
	"_animation_interval": 4,
	"_left_top_position": ( 0, 460 ),
	"_movement_status": {
		"move_left": False,
		"move_right": False,
		"jump": False,
		"burst": False,
		"straight_jump": False
	},
	"_energy": 100,
	"_normal_jump_energy": 8,
	"_massive_jump_energy": 12,
	"_movement_speed_normal": 3,
	"_jumping": False,
	"_jump_max_speed": 8,
	"_jump_increment": 0.5,
	"_jump_speed_normal": 8,
	"_jump_up": True,
	"_jump_down": False,
	"_direction": {
		"right": True,
		"left": False
	},
	"_movement_speed_burst": 6,
	"_jump_speed_burst": 12,
	"_energy_recover_rate": 5,
	"_energy_recover_trigger": 30,
	"_energy_recover_counter": 0,
	"_max_energy": 100,
	"_normal_speed_energy": 1,
	"_massive_speed_energy": 2,
	"_movement_energy": 500,
	"_movement_max_energy": 500,
	"_movement_energy_recover_rate": 2,
	"_movement_energy_recover_trigger": 2,
	"_movement_energy_recover_counter": 0,
	"_straight_jumping": False, # New - Straight jump from here.
	"_straight_jump_max_speed": 23, #14.5
	"_straight_jump_increment": 1.3, # 0.5
	"_straight_jump_speed_normal": 23, #14.5
	"_straight_jump_speed_burst": 35, #21
	"_straight_jump_up": True,
	"_straight_jump_down": False,
	"_straight_jump_energy": 100,
	"_normal_straight_jump_energy": 14, #14.
	"_massive_straight_jump_energy": 21, #21 
	"_straight_jump_energy_recover_rate": 5,
	"_straight_jump_energy_recover_trigger": 30,
	"_straight_jump_energy_recover_counter": 0,
	"_straight_jump_max_energy": 100,
	"_fireflies_decrement": 1
}


TURBINE_1 = {
	"_image_path": "turbine_1.png",
	"_left_top_position": ( 150, 150 ),
	"_cur_angle": 0,
	"_angle_amount": 0,
	"_upwards": False,
	"_downwards": True,
	"_down_limit": 400,
	"_up_limit": 100,
	"_movement_speed": 2,
	"_moving_up": False,
	"_moving_down": True,
	"_already_touched": False,
	"_can_generate_energy": False,
	"_slow_down_amount": 0.05,
	"_energy_increment": 1
}


TURBINE_2 = {
	"_image_path": "turbine_1.png",
	"_left_top_position": ( 470, 350 ),
	"_cur_angle": 0,
	"_angle_amount": 0,
	"_upwards": False,
	"_downwards": True,
	"_down_limit": 400,
	"_up_limit": 100,
	"_movement_speed": 2,
	"_moving_up": False,
	"_moving_down": True,
	"_already_touched": False,
	"_can_generate_energy": False,
	"_slow_down_amount": 0.05,
	"_energy_increment": 1
}

FIREFLY = {
	"_image_path": "moon_worm.png",
	"_min_max_vertical": ( 0, 350 ),
	"_min_max_horizontal": ( 0, 640 ),
	"_timer": 0,
	"_timer_increment": 1,
	"_min_max_stay_in_location_time": ( 30, 180 )
}

N_FIREFLIES = 7

LAMP = {
	"_images": ( "lamp_1.png", "lamp_2.png" ),
	"_center_xy_position": ( 640 / 2, 7 ),
	"_energy": 0,
	"_energy_reduction_amount": 1,
	"_timer": 0,
	"_max_timer": 60,
	"_timer_increment": 1
}

N_CLOUDS = 8

CLOUD = {
	"_image": "cloud_1.png",
	"_cur_alpha": 255,
	"_alpha_reducement": 5,
	"_top_min": 0,
	"_top_max": 50,
	"_speed_min": 1,
	"_speed_max": 2
}

RAINDROP = {
	"_image": "raindrop.png",
	"_drop_speed_increment": 0.1,
	"_cur_alpha": 255,
	"_alpha_reduction": 5 
}

ROOTAREA = {
	"_image_path": "root_area.png",
	"_pos_center_bottom": ( 320, 480 ),
	"_water_increment": 1,
	"_water_decrement": 1,
	"_max_timer": 150,
	"_timer_increment": 1
}

DEFAULT_FONT = "Ubuntu_R.ttf"

SINGLE_STAT_WATER = {
	"_text": "Moonwater",
	"_position": ( 20, 20 ),
	"_color": WHITE,
	"_bg_color": GREY,
	"_size": 11
}

SINGLE_STAT_ELECTRICITY = {
	"_text": "Electricity",
	"_position": ( 40, 20 ),
	"_color": WHITE,
	"_bg_color": GREY,
	"_size": 11
}

SINGLE_STAT_FIREFLIES = {
	"_text": "Moonsects",
	"_position": ( 60, 20 ),
	"_color": WHITE,
	"_bg_color": GREY,
	"_size": 11
}

SINGLE_STAT_JUMP_ENERGY = {
	"_text": "Jump energy",
	"_position": ( 80, 20 ),
	"_color": WHITE,
	"_bg_color": GREY,
	"_size": 11
}

SINGLE_STAT_STRAIGHT_JUMP_ENERGY = {
	"_text": "Straight jump energy",
	"_position": ( 100, 20 ),
	"_color": WHITE,
	"_bg_color": GREY,
	"_size": 11
}

SINGLE_STAT_MOVEMENT_ENERGY = {
	"_text": "Walking energy",
	"_position": ( 120, 20 ),
	"_color": WHITE,
	"_bg_color": GREY,
	"_size": 11
}

SINGLE_STAT_TIMER = {
	"_text": "Time",
	"_position": ( 20, 560 ),
	"_color": YELLOW,
	"_bg_color": GREY,
	"_size": 14
}

YOUR_TIME = {
	"_text": "Congrats!  Seconds to finish the game",
	"_position": ( 220, 140 ),
	"_color": YELLOW,
	"_bg_color": GREY,
	"_size": 20
}

MAIN_MENU = {
	"heading": {
		"font": DEFAULT_FONT,
		"font_size": 24,
		"text": "MOON BAMBOO GARDENER",
		"fgcolor": WHITE,
		"center": ( 320, 120 )
	},
	"menu_items": {
		"font": DEFAULT_FONT,
		"font_size": 14,
		"selected_color": YELLOW,
		"menu_items": {
			1: {
				"text": "START",
				"fgcolor": WHITE,
				"center": ( 320, 200 ),
				"index": 1,
				"ret_value": "start"
			},
			2: {
				"text": "END",
				"fgcolor": WHITE,
				"center": ( 320, 220 ),
				"index": 2,
				"ret_value": "end"
			}
		}
	},
	"controls": {
		pygame.K_DOWN: "next_item",
		pygame.K_UP: "prev_item",
		pygame.K_RETURN: "choose"
	}
}

GAME_OVER_MENU = {
	"heading": {
		"font": DEFAULT_FONT,
		"font_size": 24,
		"text": "GAME OVER",
		"fgcolor": WHITE,
		"center": ( 320, 120 )
	},
	"menu_items": {
		"font": DEFAULT_FONT,
		"font_size": 14,
		"selected_color": YELLOW,
		"menu_items": {
			1: {
				"text": "OK",
				"fgcolor": WHITE,
				"center": ( 320, 400 ),
				"index": 1,
				"ret_value": "ok"
			},
		}
	},
	"controls": {
		pygame.K_DOWN: "next_item",
		pygame.K_UP: "prev_item",
		pygame.K_RETURN: "choose"
	}
}


PAUSE_MENU = {
	"heading": {
		"font": DEFAULT_FONT,
		"font_size": 24,
		"text": "ABORT THE CURRENT GAME?",
		"fgcolor": WHITE,
		"center": ( 320, 120 )
	},
	"menu_items": {
		"font": DEFAULT_FONT,
		"font_size": 14,
		"selected_color": YELLOW,
		"menu_items": {
			1: {
				"text": "YES",
				"fgcolor": WHITE,
				"center": ( 320, 200 ),
				"index": 1,
				"ret_value": "yes"
			},
			2: {
				"text": "NO",
				"fgcolor": WHITE,
				"center": ( 320, 220 ),
				"index": 2,
				"ret_value": "no"
			},			
		}
	},
	"controls": {
		pygame.K_DOWN: "next_item",
		pygame.K_UP: "prev_item",
		pygame.K_RETURN: "choose"
	}
}
